@@ -3,7 +3,7 @@
  * This file is part of GtkSourceView
  *
  * Copyright (C) 2002-2005 - Paolo Maggi
- * Copyright (C) 2014, 2015 - Sébastien Wilmet <swilmet@gnome.org>
+ * Copyright (C) 2014 - Sébastien Wilmet <swilmet@gnome.org>
  *
  * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,12 +20,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gtksourceencoding.h"
-#include "gtksourceencoding-private.h"
 #include "gtksourceview-i18n.h"
 
 /**
@@ -56,7 +51,7 @@ G_DEFINE_BOXED_TYPE (GtkSourceEncoding, gtk_source_encoding,
  * Copyright (C) 2002 Red Hat, Inc.
  */
 
-typedef enum _GtkSourceEncodingIndex
+typedef enum
 {
 	GTK_SOURCE_ENCODING_ISO_8859_1,
 	GTK_SOURCE_ENCODING_ISO_8859_2,
@@ -310,8 +305,7 @@ gtk_source_encoding_lazy_init (void)
  * Gets a #GtkSourceEncoding from a character set such as "UTF-8" or
  * "ISO-8859-1".
  *
- * Returns: (nullable): the corresponding #GtkSourceEncoding, or %NULL
- * if not found.
+ * Returns: the corresponding #GtkSourceEncoding, or %NULL if not found.
  * Since: 3.14
  */
 const GtkSourceEncoding *
@@ -346,28 +340,26 @@ gtk_source_encoding_get_from_charset (const gchar *charset)
 }
 
 /**
- * gtk_source_encoding_get_all:
+ * gtk_source_encoding_foreach:
+ * @func: (scope call): the function to call for each encoding
+ * @user_data: user data to pass to the function
  *
- * Gets all encodings.
+ * Calls a function for each encoding.
  *
- * Returns: (transfer container) (element-type GtkSource.Encoding): a list of
- * all #GtkSourceEncoding's. Free with g_slist_free().
  * Since: 3.14
  */
-GSList *
-gtk_source_encoding_get_all (void)
+void
+gtk_source_encoding_foreach (GtkSourceEncodingForeachFunc func,
+			     gpointer                     user_data)
 {
-	GSList *all = NULL;
 	gint i;
 
-	for (i = GTK_SOURCE_ENCODING_LAST - 1; i >= 0; i--)
+	func (&utf8_encoding, user_data);
+
+	for (i = 0; i < GTK_SOURCE_ENCODING_LAST; i++)
 	{
-		all = g_slist_prepend (all, (gpointer) &encodings[i]);
+		func (&encodings[i], user_data);
 	}
-
-	all = g_slist_prepend (all, (gpointer) &utf8_encoding);
-
-	return all;
 }
 
 /**
@@ -495,197 +487,6 @@ gtk_source_encoding_get_name (const GtkSourceEncoding* enc)
 	return (enc->name == NULL) ? _("Unknown") : _(enc->name);
 }
 
-static GSList *
-strv_to_list (const gchar * const *enc_str)
-{
-	GSList *res = NULL;
-	gchar **p;
-
-	for (p = (gchar **)enc_str; p != NULL && *p != NULL; p++)
-	{
-		const gchar *charset = *p;
-		const GtkSourceEncoding *enc;
-
-		if (g_str_equal (charset, "CURRENT"))
-		{
-			g_get_charset (&charset);
-		}
-
-		g_return_val_if_fail (charset != NULL, NULL);
-		enc = gtk_source_encoding_get_from_charset (charset);
-
-		if (enc != NULL &&
-		    g_slist_find (res, enc) == NULL)
-		{
-			res = g_slist_prepend (res, (gpointer)enc);
-		}
-	}
-
-	return g_slist_reverse (res);
-}
-
-static GSList *
-remove_duplicates_keep_first (GSList *list)
-{
-	GSList *new_list = NULL;
-	GSList *l;
-
-	for (l = list; l != NULL; l = l->next)
-	{
-		gpointer cur_encoding = l->data;
-
-		if (g_slist_find (new_list, cur_encoding) == NULL)
-		{
-			new_list = g_slist_prepend (new_list, cur_encoding);
-		}
-	}
-
-	new_list = g_slist_reverse (new_list);
-
-	g_slist_free (list);
-	return new_list;
-}
-
-static GSList *
-remove_duplicates_keep_last (GSList *list)
-{
-	GSList *new_list = NULL;
-	GSList *l;
-
-	list = g_slist_reverse (list);
-
-	for (l = list; l != NULL; l = l->next)
-	{
-		gpointer cur_encoding = l->data;
-
-		if (g_slist_find (new_list, cur_encoding) == NULL)
-		{
-			new_list = g_slist_prepend (new_list, cur_encoding);
-		}
-	}
-
-	g_slist_free (list);
-	return new_list;
-}
-
-/*
- * _gtk_source_encoding_remove_duplicates:
- * @list: (element-type GtkSource.Encoding): a list of #GtkSourceEncoding's.
- * @removal_type: the #GtkSourceEncodingDuplicates.
- *
- * A convenience function to remove duplicated encodings in a list.
- *
- * Returns: (transfer container) (element-type GtkSource.Encoding): the new
- * start of the #GSList.
- * Since: 3.14
- */
-GSList *
-_gtk_source_encoding_remove_duplicates (GSList                      *list,
-					GtkSourceEncodingDuplicates  removal_type)
-{
-	switch (removal_type)
-	{
-		case GTK_SOURCE_ENCODING_DUPLICATES_KEEP_FIRST:
-			return remove_duplicates_keep_first (list);
-
-		case GTK_SOURCE_ENCODING_DUPLICATES_KEEP_LAST:
-			return remove_duplicates_keep_last (list);
-
-		default:
-			break;
-	}
-
-	g_return_val_if_reached (list);
-}
-
-/**
- * gtk_source_encoding_get_default_candidates:
- *
- * Gets the list of default candidate encodings to try when loading a file. See
- * gtk_source_file_loader_set_candidate_encodings().
- *
- * This function returns a different list depending on the current locale (i.e.
- * language, country and default encoding). The UTF-8 encoding and the current
- * locale encoding are guaranteed to be present in the returned list.
- *
- * Returns: (transfer container) (element-type GtkSource.Encoding): the list of
- * default candidate encodings. Free with g_slist_free().
- * Since: 3.18
- */
-GSList *
-gtk_source_encoding_get_default_candidates (void)
-{
-	const gchar *encodings_str;
-	const gchar *encodings_str_translated;
-	GVariant *encodings_variant;
-	const gchar **encodings_strv;
-	GSList *encodings_list;
-	GError *error = NULL;
-
-	/* Translators: This is the sorted list of encodings used by
-	 * GtkSourceView for automatic detection of the file encoding. You may
-	 * want to customize it adding encodings that are common in your
-	 * country, for instance the GB18030 encoding for the Chinese
-	 * translation. You may also want to remove the ISO-8859-15 encoding
-	 * (covering English and most Western European languages) if you think
-	 * people in your country will rarely use it.  "CURRENT" is a magic
-	 * value used by GtkSourceView and it represents the encoding for the
-	 * current locale, so please don't translate the "CURRENT" term.  Only
-	 * recognized encodings are used. See
-	 * https://git.gnome.org/browse/gtksourceview/tree/gtksourceview/gtksourceencoding.c#n147
-	 * for a list of supported encodings.
-	 * Keep the same format: square brackets, single quotes, commas.
-	 */
-	encodings_str = N_("['UTF-8', 'CURRENT', 'ISO-8859-15', 'UTF-16']");
-
-	encodings_str_translated = _(encodings_str);
-
-	encodings_variant = g_variant_parse (G_VARIANT_TYPE_STRING_ARRAY,
-					     encodings_str_translated,
-					     NULL,
-					     NULL,
-					     &error);
-
-	if (error != NULL)
-	{
-		const gchar * const *language_names = g_get_language_names ();
-
-		g_warning ("Error while parsing encodings list for locale %s:\n"
-			   "Translated list: %s\n"
-			   "Error message: %s",
-			   language_names[0],
-			   encodings_str_translated,
-			   error->message);
-
-		g_clear_error (&error);
-
-		encodings_variant = g_variant_parse (G_VARIANT_TYPE_STRING_ARRAY,
-						     encodings_str,
-						     NULL,
-						     NULL,
-						     &error);
-
-		g_assert_no_error (error);
-	}
-
-	encodings_strv = g_variant_get_strv (encodings_variant, NULL);
-	encodings_list = strv_to_list (encodings_strv);
-	g_free ((gpointer) encodings_strv);
-
-	/* Ensure that UTF-8 and CURRENT are present. */
-	encodings_list = g_slist_prepend (encodings_list,
-					  (gpointer) gtk_source_encoding_get_current ());
-
-	encodings_list = g_slist_prepend (encodings_list,
-					  (gpointer) &utf8_encoding);
-
-	encodings_list = _gtk_source_encoding_remove_duplicates (encodings_list,
-								 GTK_SOURCE_ENCODING_DUPLICATES_KEEP_LAST);
-
-	g_variant_unref (encodings_variant);
-	return encodings_list;
-}
-
 /**
  * gtk_source_encoding_copy:
  * @enc: a #GtkSourceEncoding.
@@ -716,3 +517,73 @@ gtk_source_encoding_free (GtkSourceEncoding *enc)
 {
 	g_return_if_fail (enc != NULL);
 }
+
+#if 0
+/* Will probably be used in the future. */
+static gboolean
+data_exists (GSList         *list,
+	     const gpointer  data)
+{
+	for (; list != NULL; list = g_slist_next (list))
+	{
+		if (list->data == data)
+		{
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+GSList *
+_gtk_source_encoding_strv_to_list (const gchar * const *enc_str)
+{
+	GSList *res = NULL;
+	gchar **p;
+
+	for (p = (gchar **)enc_str; p != NULL && *p != NULL; p++)
+	{
+		const gchar *charset = *p;
+		const GtkSourceEncoding *enc;
+
+		if (g_str_equal (charset, "CURRENT"))
+		{
+			g_get_charset (&charset);
+		}
+
+		g_return_val_if_fail (charset != NULL, NULL);
+		enc = gtk_source_encoding_get_from_charset (charset);
+
+		if (enc != NULL &&
+		    !data_exists (res, (gpointer)enc))
+		{
+			res = g_slist_prepend (res, (gpointer)enc);
+		}
+	}
+
+	return g_slist_reverse (res);
+}
+
+gchar **
+_gtk_source_encoding_list_to_strv (const GSList *enc_list)
+{
+	GSList *l;
+	GPtrArray *array;
+
+	array = g_ptr_array_sized_new (g_slist_length ((GSList *)enc_list) + 1);
+
+	for (l = (GSList *)enc_list; l != NULL; l = g_slist_next (l))
+	{
+		const GtkSourceEncoding *enc = l->data;
+		const gchar *charset = gtk_source_encoding_get_charset (enc);
+
+		g_return_val_if_fail (charset != NULL, NULL);
+
+		g_ptr_array_add (array, g_strdup (charset));
+	}
+
+	g_ptr_array_add (array, NULL);
+
+	return (gchar **)g_ptr_array_free (array, FALSE);
+}
+#endif
