@@ -22,16 +22,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gtksourcefileloader.h"
 #include "gtksourcebuffer.h"
 #include "gtksourcefile.h"
 #include "gtksourcebufferoutputstream.h"
 #include "gtksourceencoding.h"
-#include "gtksourceencoding-private.h"
 #include "gtksourceview-typebuiltins.h"
 #include "gtksourceview-i18n.h"
 
@@ -47,18 +42,6 @@
  * A file loader should be used only for one load operation, including errors
  * handling. If an error occurs, you can reconfigure the loader and relaunch the
  * operation with gtk_source_file_loader_load_async().
- *
- * Running a #GtkSourceFileLoader is an undoable action for the
- * #GtkSourceBuffer. That is, gtk_source_buffer_begin_not_undoable_action() and
- * gtk_source_buffer_end_not_undoable_action() are called, which delete the
- * undo/redo history.
- *
- * After a file loading, the buffer is reset to the contents provided by the
- * #GFile or #GInputStream, so the buffer is set as “unmodified”, that is,
- * gtk_text_buffer_set_modified() is called with %FALSE. If the contents isn't
- * saved somewhere (for example if you load from stdin), then you should
- * probably call gtk_text_buffer_set_modified() with %TRUE after calling
- * gtk_source_file_loader_load_finish().
  */
 
 #if 0
@@ -117,9 +100,6 @@ struct _GtkSourceFileLoaderPrivate
 	gpointer progress_cb_data;
 	GDestroyNotify progress_cb_notify;
 
-	/* Warning: type instances' private data are limited to a total of 64 KiB.
-	 * See the GType documentation.
-	 */
 	gchar chunk_buffer[READ_CHUNK_SIZE];
 	gssize chunk_bytes_read;
 
@@ -278,54 +258,6 @@ gtk_source_file_loader_dispose (GObject *object)
 	G_OBJECT_CLASS (gtk_source_file_loader_parent_class)->dispose (object);
 }
 
-static void
-set_default_candidate_encodings (GtkSourceFileLoader *loader)
-{
-	GSList *list;
-	GSList *l;
-	const GtkSourceEncoding *file_encoding;
-
-	/* Get first the default candidates from GtkSourceEncoding. If the
-	 * GtkSourceFile's encoding has been set by a FileLoader or FileSaver,
-	 * put it at the beginning of the list.
-	 */
-	list = gtk_source_encoding_get_default_candidates ();
-
-	if (loader->priv->file == NULL)
-	{
-		goto end;
-	}
-
-	file_encoding = gtk_source_file_get_encoding (loader->priv->file);
-
-	if (file_encoding == NULL)
-	{
-		goto end;
-	}
-
-	/* Remove file_encoding from the list, if already present, and prepend
-	 * it to the list.
-	 */
-	for (l = list; l != NULL; l = l->next)
-	{
-		const GtkSourceEncoding *cur_encoding = l->data;
-
-		if (cur_encoding == file_encoding)
-		{
-			list = g_slist_delete_link (list, l);
-
-			/* The list doesn't contain duplicates, normally. */
-			break;
-		}
-	}
-
-	list = g_slist_prepend (list, (gpointer) file_encoding);
-
-end:
-	g_slist_free (loader->priv->candidate_encodings);
-	loader->priv->candidate_encodings = list;
-}
-
 static void
 gtk_source_file_loader_constructed (GObject *object)
 {
@@ -333,7 +265,12 @@ gtk_source_file_loader_constructed (GObject *object)
 
 	if (loader->priv->file != NULL)
 	{
-		set_default_candidate_encodings (loader);
+		const GtkSourceEncoding *encoding;
+
+		encoding = gtk_source_file_get_encoding (loader->priv->file);
+
+		g_slist_free (loader->priv->candidate_encodings);
+		loader->priv->candidate_encodings = g_slist_prepend (NULL, (gpointer) encoding);
 
 		if (loader->priv->location == NULL &&
 		    loader->priv->input_stream_property == NULL)
@@ -352,7 +289,7 @@ gtk_source_file_loader_constructed (GObject *object)
 		}
 	}
 
-	G_OBJECT_CLASS (gtk_source_file_loader_parent_class)->constructed (object);
+	G_OBJECT_CLASS (gtk_source_file_loader_parent_class)->dispose (object);
 }
 
 static void
@@ -410,7 +347,7 @@ gtk_source_file_loader_class_init (GtkSourceFileLoaderClass *klass)
 	 */
 	g_object_class_install_property (object_class, PROP_LOCATION,
 					 g_param_spec_object ("location",
-							      "Location",
+							      _("Location"),
 							      "",
 							      G_TYPE_FILE,
 							      G_PARAM_READWRITE |
@@ -427,7 +364,7 @@ gtk_source_file_loader_class_init (GtkSourceFileLoaderClass *klass)
 	 */
 	g_object_class_install_property (object_class, PROP_INPUT_STREAM,
 					 g_param_spec_object ("input-stream",
-							      "Input stream",
+							      _("Input stream"),
 							      "",
 							      G_TYPE_INPUT_STREAM,
 							      G_PARAM_READWRITE |
@@ -677,9 +614,6 @@ create_input_stream (GtkSourceFileLoader *loader)
 			case GTK_SOURCE_COMPRESSION_TYPE_NONE:
 				/* NOOP */
 				break;
-
-			default:
-				g_assert_not_reached ();
 		}
 	}
 
@@ -901,14 +835,8 @@ gtk_source_file_loader_new_from_stream (GtkSourceBuffer *buffer,
  * Sets the candidate encodings for the file loading. The encodings are tried in
  * the same order as the list.
  *
- * For convenience, @candidate_encodings can contain duplicates. Only the first
- * occurrence of a duplicated encoding is kept in the list.
- *
- * By default the candidate encodings are (in that order in the list):
- * 1. If set, the #GtkSourceFile's encoding as returned by
- * gtk_source_file_get_encoding().
- * 2. The default candidates as returned by
- * gtk_source_encoding_get_default_candidates().
+ * There is by default only one candidate encoding, the #GtkSourceFile's
+ * encoding.
  *
  * Since: 3.14
  */
@@ -916,13 +844,8 @@ void
 gtk_source_file_loader_set_candidate_encodings (GtkSourceFileLoader *loader,
 						GSList              *candidate_encodings)
 {
-	GSList *list;
-
-	list = g_slist_copy (candidate_encodings);
-	list = _gtk_source_encoding_remove_duplicates (list, GTK_SOURCE_ENCODING_DUPLICATES_KEEP_FIRST);
-
 	g_slist_free (loader->priv->candidate_encodings);
-	loader->priv->candidate_encodings = list;
+	loader->priv->candidate_encodings = g_slist_copy (candidate_encodings);
 }
 
 /**
@@ -959,8 +882,8 @@ gtk_source_file_loader_get_file (GtkSourceFileLoader *loader)
  * gtk_source_file_loader_get_location:
  * @loader: a #GtkSourceFileLoader.
  *
- * Returns: (nullable) (transfer none): the #GFile to load, or %NULL
- * if an input stream is used.
+ * Returns: (transfer none): the #GFile to load, or %NULL if an input stream is
+ *   used.
  * Since: 3.14
  */
 GFile *
@@ -975,8 +898,8 @@ gtk_source_file_loader_get_location (GtkSourceFileLoader *loader)
  * gtk_source_file_loader_get_input_stream:
  * @loader: a #GtkSourceFileLoader.
  *
- * Returns: (nullable) (transfer none): the #GInputStream to load, or %NULL
- * if a #GFile is used.
+ * Returns: (transfer none): the #GInputStream to load, or %NULL if a #GFile is
+ *   used.
  * Since: 3.14
  */
 GInputStream *
@@ -1109,7 +1032,7 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 				    GError              **error)
 {
 	gboolean ok;
-	gboolean update_file_properties;
+	gboolean update_buffer_properties;
 	GError *real_error = NULL;
 
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE_LOADER (loader), FALSE);
@@ -1131,11 +1054,11 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 	 * With the other errors, normally the contents hasn't been loaded into
 	 * the buffer, i.e. the buffer is still empty.
 	 */
-	update_file_properties = ok || (real_error != NULL &&
-					real_error->domain == GTK_SOURCE_FILE_LOADER_ERROR &&
-					real_error->code == GTK_SOURCE_FILE_LOADER_ERROR_CONVERSION_FALLBACK);
+	update_buffer_properties = ok || (real_error != NULL &&
+					  real_error->domain == GTK_SOURCE_FILE_LOADER_ERROR &&
+					  real_error->code == GTK_SOURCE_FILE_LOADER_ERROR_CONVERSION_FALLBACK);
 
-	if (update_file_properties && loader->priv->file != NULL)
+	if (update_buffer_properties && loader->priv->file != NULL)
 	{
 		/* The location is already updated at the beginning of the
 		 * operation.
@@ -1150,29 +1073,12 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 		_gtk_source_file_set_compression_type (loader->priv->file,
 						       loader->priv->auto_detected_compression_type);
 
-		_gtk_source_file_set_externally_modified (loader->priv->file, FALSE);
-		_gtk_source_file_set_deleted (loader->priv->file, FALSE);
-
 		if (g_file_info_has_attribute (loader->priv->info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
 		{
 			GTimeVal modification_time;
 			g_file_info_get_modification_time (loader->priv->info, &modification_time);
 			_gtk_source_file_set_modification_time (loader->priv->file, modification_time);
 		}
-
-		if (g_file_info_has_attribute (loader->priv->info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
-		{
-			gboolean readonly;
-
-			readonly = !g_file_info_get_attribute_boolean (loader->priv->info,
-								       G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
-
-			_gtk_source_file_set_readonly (loader->priv->file, readonly);
-		}
-		else
-		{
-			_gtk_source_file_set_readonly (loader->priv->file, FALSE);
-		}
 	}
 
 	reset (loader);
