@@ -22,10 +22,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gtksourcefilesaver.h"
 #include "gtksourcefile.h"
 #include "gtksourcebufferinputstream.h"
@@ -111,9 +107,6 @@ struct _GtkSourceFileSaverPrivate
 	gpointer progress_cb_data;
 	GDestroyNotify progress_cb_notify;
 
-	/* Warning: type instances' private data are limited to a total of 64 KiB.
-	 * See the GType documentation.
-	 */
 	gchar chunk_buffer[WRITE_CHUNK_SIZE];
 	gssize chunk_bytes_read;
 	gssize chunk_bytes_written;
@@ -377,7 +370,7 @@ gtk_source_file_saver_class_init (GtkSourceFileSaverClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_LOCATION,
 					 g_param_spec_object ("location",
-							      "Location",
+							      _("Location"),
 							      "",
 							      G_TYPE_FILE,
 							      G_PARAM_READWRITE |
@@ -394,7 +387,7 @@ gtk_source_file_saver_class_init (GtkSourceFileSaverClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_ENCODING,
 					 g_param_spec_boxed ("encoding",
-							     "Encoding",
+							     _("Encoding"),
 							     "",
 							     GTK_SOURCE_TYPE_ENCODING,
 							     G_PARAM_READWRITE |
@@ -411,7 +404,7 @@ gtk_source_file_saver_class_init (GtkSourceFileSaverClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_NEWLINE_TYPE,
 					 g_param_spec_enum ("newline-type",
-					                    "Newline type",
+					                    _("Newline type"),
 							    "",
 					                    GTK_SOURCE_TYPE_NEWLINE_TYPE,
 					                    GTK_SOURCE_NEWLINE_TYPE_LF,
@@ -429,7 +422,7 @@ gtk_source_file_saver_class_init (GtkSourceFileSaverClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_COMPRESSION_TYPE,
 					 g_param_spec_enum ("compression-type",
-					                    "Compression type",
+					                    _("Compression type"),
 					                    "",
 					                    GTK_SOURCE_TYPE_COMPRESSION_TYPE,
 					                    GTK_SOURCE_COMPRESSION_TYPE_NONE,
@@ -447,7 +440,7 @@ gtk_source_file_saver_class_init (GtkSourceFileSaverClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_FLAGS,
 					 g_param_spec_flags ("flags",
-							     "Flags",
+							     _("Flags"),
 							     "",
 							     GTK_SOURCE_TYPE_FILE_SAVER_FLAGS,
 							     GTK_SOURCE_FILE_SAVER_FLAGS_NONE,
@@ -891,6 +884,9 @@ check_externally_modified_cb (GFile              *location,
 		}
 	}
 
+	/* FIXME for a "save as" it doesn't work, the mtime stored in
+	 * GtkSourceFile is not for the good file.
+	 */
 	if (_gtk_source_file_get_modification_time (saver->priv->file, &old_mtime) &&
 	    info != NULL &&
 	    g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
@@ -924,25 +920,7 @@ check_externally_modified_cb (GFile              *location,
 static void
 check_externally_modified (GtkSourceFileSaver *saver)
 {
-	gboolean save_as = FALSE;
-
-	if (saver->priv->file != NULL)
-	{
-		GFile *prev_location;
-
-		prev_location = gtk_source_file_get_location (saver->priv->file);
-
-		/* Don't check for externally modified for a "save as" operation,
-		 * because the user has normally accepted to overwrite the file if it
-		 * already exists.
-		 */
-		save_as = (prev_location == NULL ||
-			   !g_file_equal (prev_location, saver->priv->location));
-	}
-
-
-	if (saver->priv->flags & GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME ||
-	    save_as)
+	if (saver->priv->flags & GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME)
 	{
 		begin_write (saver);
 		return;
@@ -977,7 +955,6 @@ mount_cb (GFile              *file,
 	if (error != NULL)
 	{
 		g_task_return_error (saver->priv->task, error);
-		return;
 	}
 
 	check_externally_modified (saver);
@@ -1379,9 +1356,6 @@ gtk_source_file_saver_save_async (GtkSourceFileSaver     *saver,
  * properties will be updated: the location, the encoding, the newline type and
  * the compression type.
  *
- * Since the 3.20 version, gtk_text_buffer_set_modified() is called with %FALSE
- * if the file has been saved successfully.
- *
  * Returns: whether the file was saved successfully.
  * Since: 3.14
  */
@@ -1412,10 +1386,6 @@ gtk_source_file_saver_save_finish (GtkSourceFileSaver  *saver,
 		_gtk_source_file_set_compression_type (saver->priv->file,
 						       saver->priv->compression_type);
 
-		_gtk_source_file_set_externally_modified (saver->priv->file, FALSE);
-		_gtk_source_file_set_deleted (saver->priv->file, FALSE);
-		_gtk_source_file_set_readonly (saver->priv->file, FALSE);
-
 		if (g_file_info_has_attribute (saver->priv->info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
 		{
 			GTimeVal modification_time;
@@ -1424,12 +1394,6 @@ gtk_source_file_saver_save_finish (GtkSourceFileSaver  *saver,
 		}
 	}
 
-	if (ok && saver->priv->source_buffer != NULL)
-	{
-		gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (saver->priv->source_buffer),
-					      FALSE);
-	}
-
 	reset (saver);
 
 	return ok;
