@@ -23,7 +23,6 @@
 #include "gtksourcegutter-private.h"
 #include "gtksourceview.h"
 #include "gtksourceview-i18n.h"
-#include "gtksourceview-marshal.h"
 #include "gtksourcegutterrenderer.h"
 #include "gtksourcegutterrenderer-private.h"
 
@@ -124,6 +123,13 @@ static void on_view_style_updated (GtkSourceView    *view,
 static void do_redraw (GtkSourceGutter *gutter);
 static void update_gutter_size (GtkSourceGutter *gutter);
 
+static GdkWindow *
+get_window (GtkSourceGutter *gutter)
+{
+	return gtk_text_view_get_window (GTK_TEXT_VIEW (gutter->priv->view),
+	                                 gutter->priv->window_type);
+}
+
 static void
 on_renderer_size_changed (GtkSourceGutterRenderer *renderer,
                           GParamSpec              *spec,
@@ -388,16 +394,49 @@ calculate_gutter_size (GtkSourceGutter  *gutter,
 	return total_width;
 }
 
+static void
+window_invalidate_handler (GdkWindow      *window,
+			   cairo_region_t *region)
+{
+	cairo_rectangle_int_t rect;
+
+	/* Always invalidate the whole window.
+	 * When the text is modified in a GtkTextBuffer, GtkTextView tries to
+	 * redraw the smallest required region. But the information displayed in
+	 * the gutter may become invalid in a bigger region.
+	 * See https://bugzilla.gnome.org/show_bug.cgi?id=732418 for an example
+	 * where line numbers are not updated correctly when splitting a wrapped
+	 * line.
+	 * The performances should not be a big problem here. Correctness is
+	 * more important than performances.
+	 */
+
+	rect.x = 0;
+	rect.y = 0;
+	rect.width = gdk_window_get_width (window);
+	rect.height = gdk_window_get_height (window);
+
+	cairo_region_union_rectangle (region, &rect);
+}
+
 static void
 update_gutter_size (GtkSourceGutter *gutter)
 {
 	gint width;
+	GdkWindow *window;
 
 	width = calculate_gutter_size (gutter, NULL);
 
 	gtk_text_view_set_border_window_size (GTK_TEXT_VIEW (gutter->priv->view),
 	                                      gutter->priv->window_type,
 	                                      width);
+
+	window = get_window (gutter);
+
+	if (window != NULL)
+	{
+		gdk_window_set_invalidate_handler (window, window_invalidate_handler);
+	}
 }
 
 static gboolean
@@ -614,13 +653,6 @@ gtk_source_gutter_new (GtkSourceView     *view,
 	                     NULL);
 }
 
-static GdkWindow *
-get_window (GtkSourceGutter *gutter)
-{
-	return gtk_text_view_get_window (GTK_TEXT_VIEW (gutter->priv->view),
-	                                 gutter->priv->window_type);
-}
-
 /* Public API */
 
 /**
@@ -906,7 +938,6 @@ on_view_draw (GtkSourceView   *view,
 	GtkTextView *text_view;
 	GArray *sizes;
 	GdkRectangle clip;
-	gint x, y;
 	gint y1, y2;
 	GArray *numbers;
 	GArray *pixels;
@@ -927,8 +958,6 @@ on_view_draw (GtkSourceView   *view,
 	gint idx;
 	GtkStyleContext *style_context;
 	GdkRGBA fg_color;
-	GdkDeviceManager *device_manager;
-	GdkDevice *pointer;
 
 	window = get_window (gutter);
 
@@ -950,10 +979,6 @@ on_view_draw (GtkSourceView   *view,
 
 	buffer = gtk_text_view_get_buffer (text_view);
 
-	device_manager = gdk_display_get_device_manager (gdk_window_get_display (window));
-	pointer = gdk_device_manager_get_client_pointer (device_manager);
-	gdk_window_get_device_position (window, pointer, &x, &y, NULL);
-
 	y1 = clip.y;
 	y2 = y1 + clip.height;
 
@@ -979,9 +1004,6 @@ on_view_draw (GtkSourceView   *view,
 
 	calculate_gutter_size (gutter, sizes);
 
-	i = 0;
-	x = 0;
-
 	background_area.x = 0;
 	background_area.height = get_lines (text_view,
 	                                    y1,
@@ -1005,9 +1027,6 @@ on_view_draw (GtkSourceView   *view,
 
 	cell_area.y = background_area.y;
 
-	item = gutter->priv->renderers;
-	idx = 0;
-
 	style_context = gtk_widget_get_style_context (GTK_WIDGET (view));
 
 	gtk_style_context_get_color (style_context,
@@ -1016,13 +1035,15 @@ on_view_draw (GtkSourceView   *view,
 
 	gdk_cairo_set_source_rgba (cr, &fg_color);
 
-	while (item)
+	for (item = gutter->priv->renderers, idx = 0;
+	     item != NULL;
+	     item = g_list_next (item), idx++)
 	{
 		Renderer *renderer = item->data;
 		gint xpad;
 		gint width;
 
-		width = g_array_index (sizes, gint, idx++);
+		width = g_array_index (sizes, gint, idx);
 
 		if (gtk_source_gutter_renderer_get_visible (renderer->renderer))
 		{
@@ -1051,8 +1072,6 @@ on_view_draw (GtkSourceView   *view,
 
 			background_area.x += background_area.width;
 		}
-
-		item = g_list_next (item);
 	}
 
 	gtk_text_buffer_get_iter_at_mark (buffer,
@@ -1105,9 +1124,9 @@ on_view_draw (GtkSourceView   *view,
 		background_area.height = g_array_index (heights, gint, i);
 		background_area.x = 0;
 
-		idx = 0;
-
-		for (item = gutter->priv->renderers; item; item = g_list_next (item))
+		for (item = gutter->priv->renderers, idx = 0;
+		     item != NULL;
+		     item = g_list_next (item), idx++)
 		{
 			Renderer *renderer;
 			gint width;
@@ -1116,7 +1135,7 @@ on_view_draw (GtkSourceView   *view,
 			gint ypad;
 
 			renderer = item->data;
-			width = g_array_index (sizes, gint, idx++);
+			width = g_array_index (sizes, gint, idx);
 
 			if (!gtk_source_gutter_renderer_get_visible (renderer->renderer))
 			{
@@ -1183,7 +1202,7 @@ on_view_draw (GtkSourceView   *view,
 		gtk_text_iter_forward_line (&start);
 	}
 
-	for (item = gutter->priv->renderers; item; item = g_list_next (item))
+	for (item = gutter->priv->renderers; item != NULL; item = g_list_next (item))
 	{
 		Renderer *renderer = item->data;
 
@@ -1196,7 +1215,6 @@ on_view_draw (GtkSourceView   *view,
 	g_array_free (numbers, TRUE);
 	g_array_free (pixels, TRUE);
 	g_array_free (heights, TRUE);
-
 	g_array_free (sizes, TRUE);
 
 	gutter->priv->is_drawing = FALSE;
