@@ -51,26 +51,6 @@ struct _GtkSourceSearchContextClass
 	gpointer padding[10];
 };
 
-/**
- * GtkSourceRegexSearchState:
- * @GTK_SOURCE_REGEX_SEARCH_NO_ERROR: everything is fine, or the regex search is
- * disabled.
- * @GTK_SOURCE_REGEX_SEARCH_COMPILATION_ERROR: pattern compilation error.
- * @GTK_SOURCE_REGEX_SEARCH_MATCHING_ERROR: error while matching the buffer.
- * @GTK_SOURCE_REGEX_SEARCH_REPLACE_ERROR: replace error.
- *
- * Regular expression search state.
- *
- * Since: 3.10
- */
-typedef enum
-{
-	GTK_SOURCE_REGEX_SEARCH_NO_ERROR,
-	GTK_SOURCE_REGEX_SEARCH_COMPILATION_ERROR,
-	GTK_SOURCE_REGEX_SEARCH_MATCHING_ERROR,
-	GTK_SOURCE_REGEX_SEARCH_REPLACE_ERROR
-} GtkSourceRegexSearchState;
-
 GType			 gtk_source_search_context_get_type			(void) G_GNUC_CONST;
 
 GtkSourceSearchContext	*gtk_source_search_context_new				(GtkSourceBuffer	 *buffer,
@@ -90,8 +70,6 @@ void			 gtk_source_search_context_set_highlight		(GtkSourceSearchContext  *searc
 
 GError			*gtk_source_search_context_get_regex_error		(GtkSourceSearchContext	 *search);
 
-GtkSourceRegexSearchState gtk_source_search_context_get_regex_state		(GtkSourceSearchContext	 *search);
-
 gint			 gtk_source_search_context_get_occurrences_count	(GtkSourceSearchContext	 *search);
 
 gint			 gtk_source_search_context_get_occurrence_position	(GtkSourceSearchContext	 *search,
@@ -136,11 +114,13 @@ gboolean		 gtk_source_search_context_replace			(GtkSourceSearchContext	 *search,
 										 const GtkTextIter	 *match_start,
 										 const GtkTextIter	 *match_end,
 										 const gchar		 *replace,
-										 gint			  replace_length);
+										 gint			  replace_length,
+										 GError			**error);
 
 guint			 gtk_source_search_context_replace_all			(GtkSourceSearchContext	 *search,
 										 const gchar		 *replace,
-										 gint			  replace_length);
+										 gint			  replace_length,
+										 GError			**error);
 
 G_GNUC_INTERNAL
 void			 _gtk_source_search_context_update_highlight		(GtkSourceSearchContext	 *search,
